@@ -4,9 +4,10 @@
 
                  Copyright (C) 2005, 2006 NetCitadel, LLC
 
-  Author:  Illiya Yalovoy <yalovoy@gmail.com>
+  Author:  Vadim Kurland     vadim@fwbuilder.org
+           Illiya Yalovoy    <yalovoy@gmail.com>
 
-  $Id: DiscoveryDruid.cpp,v 1.27 2006/09/18 06:59:52 vkurland Exp $
+  $Id: DiscoveryDruid.cpp,v 1.37 2007/06/13 02:58:48 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,6 +37,7 @@
 #include <qtimer.h>
 #include <qregexp.h>
 #include <qlabel.h>
+#include <qtextbrowser.h>
 #include <qtextedit.h>
 #include <qpushbutton.h>
 #include <qfile.h>
@@ -71,11 +73,14 @@
 #include "ObjectManipulator.h"
 #include "FWWindow.h"
 
+#include "IOSImporter.h"
+#include "IPTImporter.h"
 
 using namespace std;
 using namespace libfwbuilder;
 
-DiscoveryDruid::DiscoveryDruid(QWidget *parent) : DiscoveryDruid_q(parent)
+DiscoveryDruid::DiscoveryDruid(QWidget *parent, bool start_with_import) :
+    DiscoveryDruid_q(parent)
 {
     thread=NULL;
     
@@ -120,11 +125,31 @@ DiscoveryDruid::DiscoveryDruid(QWidget *parent) : DiscoveryDruid_q(parent)
     dm_usesnmp->setEnabled(false);
 #endif
 
+#ifndef HAVE_ANTLR_RUNTIME
+    dm_import_config->setEnabled(false);
+#endif
+
     dns=NULL;
     
     restore();
+
+    importPlatformChanged(import_platform->currentItem());
+
+#ifdef HAVE_ANTLR_RUNTIME
+    if (start_with_import)
+    {
+        dm_import_config->setDown(true);
+        setDiscoveryMethod_Import();
+        setAppropriate( QWizard::page(0), false );
+        // show the first page of the "import policy" track of the wizard
+        showPage( QWizard::page(2) );
+        cancelButton()->show();
+    }
+#endif
+
     prg_timer->start(100);
 }
+
 DiscoveryDruid::~DiscoveryDruid()
 {
     delete flt_obj;
@@ -137,6 +162,7 @@ DiscoveryDruid::~DiscoveryDruid()
 
     save();
 }
+
 const char * DISCOVERY_DRUID_PREFIX="DiscoveryDruid/";
 
 const char * DISCOVERY_DRUID_DISCOVERYMETHOD="DiscoveryMethod";
@@ -159,6 +185,7 @@ const char * DISCOVERY_DRUID_SNMPTIMEOUT    ="SNMPTimeout";
 const char * DISCOVERY_DRUID_SNMPDNSRETRIES ="DNSRetries";
 const char * DISCOVERY_DRUID_SNMPDNSTIMEOUT ="DNSTimeout";
 const char * DISCOVERY_DRUID_SNMPDNSTHREADS ="SNMPDnsThreads";
+const char * DISCOVERY_DRUID_IMPORRT_CONFIG_PLATFORM = "ImportPlatform";
 
 
 void DiscoveryDruid::restore()
@@ -169,6 +196,7 @@ void DiscoveryDruid::restore()
     dm_buttonGroup->setButton(st->getInt(
                 QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_DISCOVERYMETHOD));
     changedDiscoveryMethod(dm_buttonGroup->id(dm_buttonGroup->selected()));
+
     //filename->setText(st->getStr(
     //            QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_FILENAME));
     //domainname->setText(st->getStr(
@@ -207,6 +235,9 @@ void DiscoveryDruid::restore()
     snmpdnstimeout->setValue((i)?i:2);
     i=st->getInt(QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPDNSTHREADS);
     snmpdnsthreads->setValue((i)?i:5);
+
+    import_platform->setCurrentItem(st->getInt(
+           QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_IMPORRT_CONFIG_PLATFORM));
 }
 
 void DiscoveryDruid::save()
@@ -218,7 +249,7 @@ void DiscoveryDruid::save()
     st->setBool(
             QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_USELONGNAME,
             uselongname->isChecked());
-    if (Task==BT_DNS)
+    if (current_task==BT_DNS)
     {
         st->setInt(
                 QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_DNSTIMEOUT,
@@ -269,6 +300,10 @@ void DiscoveryDruid::save()
     st->setInt(
             QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_SNMPDNSTHREADS,
             snmpdnsthreads->value());
+    st->setInt(
+        QString(DISCOVERY_DRUID_PREFIX) + DISCOVERY_DRUID_IMPORRT_CONFIG_PLATFORM,
+        import_platform->currentItem());
+
 }
 
 void DiscoveryDruid::dnsFinish()
@@ -282,7 +317,7 @@ void DiscoveryDruid::dnsFinish()
     if (userIsTyping)
     {
         //abandon the test result
-        if (Task==BT_DNS)
+        if (current_task==BT_DNS)
         {
             changedNameServer();
         }
@@ -314,117 +349,133 @@ void DiscoveryDruid::dnsFinish()
 
 void DiscoveryDruid::changedSelected( const QString &s )
 {
-    QWidget *w=currentPage();
-    int page=indexOf(w);
+    QWidget *w = currentPage();
+    int page = indexOf(w);
     switch (page)
     {
     
     case 1: // Reading file in hosts format 
-        {
-            setNextEnabled(w,false);
-            changedHostsFileName();
-            filename->setFocus();
-            
-            break;
-        }
-    case 2: // Import DNS zone
-        {
-            changedDomainName();
-            domainname->setFocus();
-            //setNextEnabled(w,false);
-            break;
-        }
-    case 3: // Name server
-        {
-            if (page>FromPage)
-                getNameServers();
-            disconnect(timer,SIGNAL(timeout()),0,0);
-            connect(timer,SIGNAL(timeout()),this,SLOT(checkHostName()));
-            changedNameServer();
-            nameserverline->setFocus();
+    {
+        setNextEnabled(w,false);
+        changedHostsFileName();
+        filename->setFocus();
+        break;
+    }
 
-            //setNextEnabled(w,false);
-            break;
-        }
-    case 4: // Network discovery using SNMP
-        {
-            disconnect(timer,SIGNAL(timeout()),0,0);
-            connect(timer,SIGNAL(timeout()),this,SLOT(checkHostName()));
+    case 2: // import config
+    {
+        import_filename->setFocus();
+        setBackEnabled(w,true);
+        setFinishEnabled(w,false);
+        break;
+    }
 
-            isSeedHostOK=false;
-            isSNMPInclNetOK=false;
+    case 3: // Import DNS zone
+    {
+        changedDomainName();
+        domainname->setFocus();
+        //setNextEnabled(w,false);
+        break;
+    }
+
+    case 4: // Name server
+    {
+        if (page>FromPage)
+            getNameServers();
+        disconnect(timer,SIGNAL(timeout()),0,0);
+        connect(timer,SIGNAL(timeout()),this,SLOT(checkHostName()));
+        changedNameServer();
+        nameserverline->setFocus();
+
+        //setNextEnabled(w,false);
+        break;
+    }
+
+    case 5: // Network discovery using SNMP
+    {
+        disconnect(timer,SIGNAL(timeout()),0,0);
+        connect(timer,SIGNAL(timeout()),this,SLOT(checkHostName()));
+
+        isSeedHostOK=false;
+        isSNMPInclNetOK=false;
             
-            changedSeedHost();
-            changedInclNet();
-            seedhostname->setFocus();
-            break;
-        }
-    case 5: // Network scan options
-        {
-            snmprecursive->setFocus();
-            //setNextEnabled(w,false);
-            break;
-        }
-    case 6: // SNMP and DNS reverse lookup queries parameters
-        {
-            checkSNMPCommunity();
-            snmpcommunity->setFocus();
-            break;
-        }
-    case 7: // Background process
-        {
-            discoveryprogress->setProgress(-1);
-            discoverylog->clear();
-            discoveryStopButton->setEnabled(true);
-            logSaveButton->setEnabled(false);
-            setNextEnabled(w,false);
-            cancelButton()->hide();
+        changedSeedHost();
+        changedInclNet();
+        seedhostname->setFocus();
+        break;
+    }
+
+    case 6: // Network scan options
+    {
+        snmprecursive->setFocus();
+        //setNextEnabled(w,false);
+        break;
+    }
+
+    case 7: // SNMP and DNS reverse lookup queries parameters
+    {
+        checkSNMPCommunity();
+        snmpcommunity->setFocus();
+        break;
+    }
+
+    case 8: // Background process (import from hosts and from config file)
+    {
+        discoveryprogress->setProgress(-1);
+        discoverylog->clear();
+        discoveryStopButton->setEnabled(true);
+        logSaveButton->setEnabled(false);
+        setNextEnabled(w,false);
+        cancelButton()->hide();
+        setBackEnabled(w,false);
+        disconnect(timer,SIGNAL(timeout()),0,0);
+        connect(timer,SIGNAL(timeout()),this,SLOT(updateLog()));
+        timer->start(1000,false);
+        startBackgroundProcess();
+        break;
+    }
+
+    case 9: // Networks
+    {
+        fillListOfNetworks();
+        fillNetworks();
+        backButton()->setEnabled(false);
+        nextButton()->setEnabled(networklist->count ()>0 || Objects.size()>0);
+        break;
+    }
+
+    case 10: // Objects
+    {
+        if (Networks.size()==0)
             setBackEnabled(w,false);
-            disconnect(timer,SIGNAL(timeout()),0,0);
-            connect(timer,SIGNAL(timeout()),this,SLOT(updateLog()));
-            timer->start(1000,false);
             
-            startBackgroundProcess();
-            
-            break;
-        }
-    case 8: // Networks
-        {
-            fillListOfNetworks();
-            fillNetworks();
-            backButton()->setEnabled(false);
-            nextButton()->setEnabled(networklist->count ()>0 || Objects.size()>0);
-            break;
-        }
-    case 9: // Objects
-        {
-            if (Networks.size()==0)
-                setBackEnabled(w,false);
-            
-            fillListOfObjects();
-            fillObjects();
-            nextButton()->setEnabled(objectlist->count ()>0 || networklist->count()>0);
-            break;
-        }
-    case 10: // Adjust Object type
-        {
-            setBackEnabled(w,true);
-            fillTypeChangingList();
-            break;
-        }
-    case 11: // Target library
-        {
-            break;
-        }
-    case 12: // Objects creation ...
-        {
-            setBackEnabled(w,false);
-            cancelButton()->hide();
-            createRealObjects();
-            setFinishEnabled(w,true);
-            finishButton()->setFocus();
-            break;
-        }
+        fillListOfObjects();
+        fillObjects();
+        nextButton()->setEnabled(objectlist->count ()>0 || networklist->count()>0);
+        break;
+    }
+
+    case 11: // Adjust Object type
+    {
+        setBackEnabled(w,true);
+        fillTypeChangingList();
+        break;
+    }
+
+    case 12: // Target library
+    {
+        break;
+    }
+
+    case 13: // Objects creation ...
+    {
+        setBackEnabled(w,false);
+        cancelButton()->hide();
+        createRealObjects();
+        setFinishEnabled(w,true);
+        finishButton()->setFocus();
+        break;
+    }
         
     default : {}
     
@@ -434,33 +485,38 @@ void DiscoveryDruid::changedSelected( const QString &s )
 
 void DiscoveryDruid::startBackgroundProcess()
 {
-    switch (Task)
+    switch (current_task)
     {
         case BT_HOSTS:
+        case BT_IMPORT:
         {
             discoveryprogress->setTotalSteps(100);
             discoveryprogress->setProgress(0);
+            discoveryprogress->setEnabled(false);
             discoveryStopButton->setEnabled(false);
-            startHostsScan();
             break;
         }
         case BT_DNS:
-        {
-            discoveryprogress->setTotalSteps(0);
-            discoveryprogress->setProgress(-1);
-            startDNSScan();
-            break;
-        }
         case BT_SNMP:
         {
             discoveryprogress->setTotalSteps(0);
             discoveryprogress->setProgress(-1);
-            startSNMPScan();
             break;
         }
         default:
         {}
     }
+
+    switch (current_task)
+    {
+        case BT_HOSTS:    startHostsScan();     break;
+        case BT_DNS:      startDNSScan();       break;
+        case BT_SNMP:     startSNMPScan();      break;
+        case BT_IMPORT:   startConfigImport();  break;
+        default:
+        {}
+    }
+
 }
 
 void DiscoveryDruid::browseHostsFile()
@@ -484,6 +540,27 @@ void DiscoveryDruid::browseHostsFile()
     
 }
 
+void DiscoveryDruid::browseForImport()
+{
+    QString dir;
+    dir=st->getWDir();
+    if (dir.isEmpty())  dir=st->getOpenFileDir();
+    if (dir.isEmpty())  dir="~";
+    
+    QString s = QFileDialog::getOpenFileName(
+                    dir,
+                    "All files (*.*)",
+                    this,
+                    "open file dialog",
+                    "Choose a file" );
+    
+    if (!s.isEmpty())
+    {
+        import_filename->setText(s);
+    }
+    
+}
+
 void DiscoveryDruid::updatePrg()
 {
     if (unBar!=NULL)
@@ -493,17 +570,6 @@ void DiscoveryDruid::updatePrg()
     
 }
 
-void DiscoveryDruid::setDiscoveryMethod_file()
-{
-    Task=BT_HOSTS;
-    processname->setText(tr("Hosts file parsing ..."));
-    for (int i=0;i<WIZARD_PAGES;i++)
-    {
-        
-        setAppropriate(page(i),WIZARD_FILE_PAGES[i]);
-    }
-}
-
 void DiscoveryDruid::getNameServers()
 {
     multimap<string,libfwbuilder::IPAddress> ns_records;
@@ -544,47 +610,75 @@ void DiscoveryDruid::getNameServers()
     }
 }
 
-void DiscoveryDruid::setDiscoveryMethod_DNS()
+void DiscoveryDruid::setDiscoveryMethod_file()
 {
-    processname->setText(tr("DNS zone transfer ..."));
-    Task=BT_DNS;
+    current_task=BT_HOSTS;
+    processname->setText(tr("Hosts file parsing ..."));
     for (int i=0;i<WIZARD_PAGES;i++)
     {
-        
-        setAppropriate(page(i),WIZARD_DNS_PAGES[i]);
+        setAppropriate( QWizard::page(i), WIZARD_FILE_PAGES[i]);
     }
 }
 
-void DiscoveryDruid::setDiscoveryMethod_SNTP()
+void DiscoveryDruid::setDiscoveryMethod_DNS()
 {
-    processname->setText(tr("Network discovery using SNMP ..."));
-    Task=BT_SNMP;
+    processname->setText(tr("DNS zone transfer ..."));
+    current_task=BT_DNS;
     for (int i=0;i<WIZARD_PAGES;i++)
     {
-        
-        setAppropriate(page(i),WIZARD_SNMP_PAGES[i]);
+        setAppropriate( QWizard::page(i), WIZARD_DNS_PAGES[i]);
+    }
+}
+
+void DiscoveryDruid::setDiscoveryMethod_SNMP()
+{
+    processname->setText(tr("Network discovery using SNMP ..."));
+    current_task=BT_SNMP;
+    for (int i=0;i<WIZARD_PAGES;i++)
+    {
+        setAppropriate( QWizard::page(i), WIZARD_SNMP_PAGES[i]);
     }    
 }
 
+void DiscoveryDruid::setDiscoveryMethod_Import()
+{
+    processname->setText(tr("Import configuration from file ..."));
+    current_task=BT_IMPORT;
+    for (int i=0;i<WIZARD_PAGES;i++)
+    {
+        setAppropriate( QWizard::page(i), WIZARD_IMPORT_PAGES[i]);
+    }    
+}
+
+
 void DiscoveryDruid::changedDiscoveryMethod(int c)
 {
-   switch (c)
-   {
-       case 0: 
-           {
-               setDiscoveryMethod_file();
-               break;
-           }
-       case 1: 
-           {
-               setDiscoveryMethod_DNS();
-               break;
-           }
-       case 2: 
-           {
-               setDiscoveryMethod_SNTP();
-               break;
-           }
+    if (fwbdebug)
+        qDebug("DiscoveryDruid::changedDiscoveryMethod c=%d",c);
+
+    switch (c)
+    {
+    case 0: 
+    {
+        setDiscoveryMethod_file();
+        break;
+    }
+    case 1: 
+    {
+        setDiscoveryMethod_DNS();
+        break;
+    }
+    case 2: 
+    {
+        setDiscoveryMethod_SNMP();
+        break;
+    }
+    case 3:
+    {
+        setDiscoveryMethod_Import();
+        break;
+    }
+
        default : {}
    }
 }
@@ -642,12 +736,62 @@ void DiscoveryDruid::startHostsScan()
         delete thread;
     }
     
-    thread=new WorkerThread(BT_HOSTS);
-    thread->setFileName( filename->text() );
+    thread = new HostsFileImport(filename->text());
     thread->setTargetWidget(this);
-        
     thread->start();
-    
+}
+
+void DiscoveryDruid::startConfigImport()
+{
+    if (thread!=NULL)
+    {
+        delete thread;
+    }
+
+    QFile cf( import_filename->text()  );
+    if (cf.open( IO_ReadOnly ) )
+    {
+        QTextStream stream(&cf);
+        QString s = stream.read();
+        cf.close();
+        std::string *buffer = new std::string( s.latin1() );
+        //if (fwbdebug) qDebug(buffer->c_str());
+
+        // count lines, gather some general stats on the config file.
+
+        std::string::size_type pos, n;
+        pos = 0;
+        int line_count = 0;
+        while ( (n=buffer->find('\n', pos))!=std::string::npos)
+        {
+            line_count++;
+            pos = n+1;
+        }
+        discoveryprogress->setTotalSteps(line_count);
+
+        // need to pick right platform string based on
+        // import_platform->currentItem() 
+        string platform = "";
+        switch (import_platform->currentItem())
+        {
+        case 0: platform = "iosacl"; break;
+        case 1: platform = "iptables"; break;
+        }
+
+        //
+        // ConfigImport "owns" buffer - it is deleted
+        // in destructor of ConfigImport
+        //
+        thread = new ConfigImport(buffer, platform);
+        thread->setTargetWidget(this);
+        thread->start();
+    } else
+    {
+        QMessageBox::critical(this, tr("Discovery error"),
+          tr("Could not open file %1").arg(import_filename->text()));
+        QWidget *w = currentPage();
+        setBackEnabled(w,true);
+    }
 }
 
 IPAddress DiscoveryDruid::getNS()
@@ -1206,7 +1350,9 @@ void DiscoveryDruid::loadDataFromFile()
 {
     objectresultlist->clear();
     int t=0;
-    int count = thread->hosts.size();
+    HostsFileImport *himport = dynamic_cast<HostsFileImport*>(thread);
+    assert(himport!=NULL);
+    int count = himport->hosts.size();
     if (count > 0)
     {
         int upd_max=(count > 10)?count/10:1;
@@ -1216,9 +1362,7 @@ void DiscoveryDruid::loadDataFromFile()
         QProgressDialog pd(tr("Prepare objects ..."), tr("Cancel"), count,this,0,true);
         
         vector<ObjectDescriptor>::iterator i;
-        for(i=thread->hosts.begin();
-            i!=thread->hosts.end();
-            ++i)
+        for(i = himport->hosts.begin(); i != himport->hosts.end(); ++i)
         {
             if (i->type.empty())
             {
@@ -1243,6 +1387,27 @@ void DiscoveryDruid::loadDataFromFile()
     }
 }
 
+void DiscoveryDruid::loadDataFromImporter()
+{
+    ConfigImport *confimp = dynamic_cast<ConfigImport*>(thread);
+    assert(confimp!=NULL);
+    Importer *imp = confimp->getImporterObject();
+    if (imp!=NULL)
+    {
+        Firewall *fw = imp->finalize();
+
+        om->loadObjects();
+
+        if (fw)
+        {
+            om->updateObjName(fw,"", false);
+            mw->addFirewallToList(fw);
+            mw->showFirewall(fw);
+            om->editObject(fw);
+        }
+    }
+}
+
 void DiscoveryDruid::loadDataFromCrawler()
 {
 #ifdef HAVE_LIBSNMP
@@ -1409,6 +1574,8 @@ void DiscoveryDruid::fillListOfObjects()
     
 void DiscoveryDruid::customEvent(QCustomEvent *event)
 {
+    QWidget *w = currentPage();
+
     int evtype=(int)event->type();
     if (evtype == ProgressEv)
     {
@@ -1416,7 +1583,6 @@ void DiscoveryDruid::customEvent(QCustomEvent *event)
         discoveryprogress->setProgress(e->value);
     } else if (evtype == DoneEv)
     {
-        
         cancelButton()->show();
         
         timer->stop();
@@ -1424,47 +1590,67 @@ void DiscoveryDruid::customEvent(QCustomEvent *event)
         
         updateLog();
         logSaveButton->setEnabled(true);
-        loadDataFromFile();
-        
+
+        // actually create objects
+        switch (current_task)
+        {
+        case BT_HOSTS:
+            loadDataFromFile();
+            break;
+        case BT_IMPORT:
+            loadDataFromImporter();
+            break;
+        default:
+            break;
+        }
+
         thread->wait();
-        QString er=thread->getError();
+        QString er = thread->getError();
         delete thread;
         thread=NULL;
         
-        if (!er.isEmpty())
+        switch (current_task)
         {
-            QMessageBox::critical(this,tr("Discovery error"), er);
-        }
-        if (Objects.size()>0)
-        {
-            nextButton()->setDefault(true);
-            nextButton()->setFocus();
-            nextButton()->setEnabled(true);
-            backButton()->setEnabled(false);
-        }
-        else
-        {
-            backButton()->setEnabled(true);
-            nextButton()->setEnabled(false); 
+        case BT_HOSTS:
+            if (Objects.size()>0)
+            {
+                nextButton()->setDefault(true);
+                nextButton()->setFocus();
+                nextButton()->setEnabled(true);
+                backButton()->setEnabled(false);
+            }
+            else
+            {
+                backButton()->setEnabled(true);
+                nextButton()->setEnabled(false); 
+            }
+            break;
+        case BT_IMPORT:
+            setFinishEnabled(w,true);
+            finishButton()->setFocus();
+            break;
+        default:
+            break;
         }
+
     }
 }
 
 void DiscoveryDruid::updateLog()
 {
-    if (Task==BT_HOSTS)
+    if (current_task==BT_HOSTS || current_task==BT_IMPORT)
     {
         QString buf;
         if (thread!=NULL)
         {
             while(thread->Log->ready())
             {
-                buf=thread->Log->getLine().c_str();
-                discoverylog->append(buf+"\n");
+                buf = thread->Log->getLine().c_str();
+                discoverylog->insert(buf);
             }
         }
     }
-    else if (Task==BT_SNMP)
+    else if (current_task==BT_SNMP)
     {
        if (monitorOperation() > 0)
        {
@@ -1491,7 +1677,7 @@ void DiscoveryDruid::updateLog()
             if (Objects.size()>0 || Networks.size()>0)
             {
                 if (Networks.size()==0) 
-                    setAppropriate(page(8),0);
+                    setAppropriate( QWizard::page(8),0);
                 nextButton()->setEnabled(true);
                 nextButton()->setDefault(true);
                 nextButton()->setFocus();
@@ -1513,7 +1699,7 @@ void DiscoveryDruid::updateLog()
             discoveryStopButton->setEnabled(false);
        }
     }
-    else if (Task==BT_DNS)
+    else if (current_task==BT_DNS)
     {
        if (monitorOperation() > 0)
        {
@@ -1931,6 +2117,38 @@ void DiscoveryDruid::autorename(FWObject *obj,
     ol.clear();
 }
 
+void DiscoveryDruid::importPlatformChanged(int cp)
+{
+    if (fwbdebug)
+        qDebug("DiscoveryDruid::importPlatformChanged(): %d",cp);
+
+    switch (cp)
+    {
+    case 0:
+        import_text->setText(
+            QObject::tr("Firewall Builder can import Cisco IOS access lists "
+                        "from the router configuration saved using 'show run' "
+                        "or any other command that saves running config. The name "
+                        "of the created firewall object, all of its interfaces "
+                        "and their addresses will be configured automatically if "
+                        "this information can be found in the configuration file."
+            )
+        );
+        break;
+    case 1:
+        import_text->setText(
+            QObject::tr("Firewall Builder can import iptables rules "
+                        "from a file in iptables-save format. Firewall "
+                        "name and addresses of its interfaces need "
+                        "to be configured manually because iptables-save "
+                        "file does not have this information. "
+            )
+        );
+        break;
+    }
+
+}
+
 //----------------------------------------------------------------------
 ObjectDescriptor::ObjectDescriptor() {}
 
@@ -1985,24 +2203,18 @@ ObjectDescriptor& ObjectDescriptor::operator=(const ObjectDescriptor& od) {
     return *this;
 }
 
-WorkerThread::WorkerThread ( BackgroundTask t) : QThread()
+// ================================================================
+
+WorkerThread::WorkerThread() : QThread()
 {
-    Task=t;
     Log=new QueueLogger();
 }
 
-WorkerThread::~WorkerThread ()
+WorkerThread::~WorkerThread()
 {
     delete Log;
 }
 
-void WorkerThread::setFileName(const QString &s)
-{
-    FileName=s;
-}
-
-
-
 void WorkerThread::setProgress(int p)
 {
    ProgressEvent *event=new ProgressEvent();
@@ -2020,12 +2232,26 @@ void WorkerThread::done()
 
 QString WorkerThread::getError()
 {
-    return Error;
+    return last_error;
 }
-void WorkerThread::processHosts()
+
+void WorkerThread::run()
+{
+    done();
+}
+
+// ================================================================
+
+HostsFileImport::HostsFileImport(const QString &f) : 
+    WorkerThread()
+{
+    file_name = f;
+}
+
+void HostsFileImport::run()
 {
     *Log << "Discovery method:"
-         << "Read file in hosts format.";
+         << "Read file in hosts format. \n";
     
     map<IPAddress, vector<string> > reverse_hosts;
     HostsFile *hf;
@@ -2033,28 +2259,29 @@ void WorkerThread::processHosts()
  *    read hosts file here 
  */
     hf=new HostsFile();
-    Error="";
+    last_error="";
     setProgress(10);
     
-    *Log << "Parsing file: " << FileName.latin1();
-    if (!FileName.isEmpty())
+    *Log << "Parsing file: " << file_name.latin1() << "\n";
+    if (!file_name.isEmpty())
     {
         try 
         {
-            hf->parse( FileName.ascii() );
+            hf->parse( file_name.ascii() );
         } catch ( FWException &ex ) 
         {
-            Error=ex.toString().c_str();
-            *Log << "Exception: " << Error.ascii();
+            last_error = ex.toString().c_str();
+            *Log << "Exception: " << last_error.ascii() << "\n";
             
             delete hf;
+            done();
             return;
         }
         reverse_hosts=hf->getAll();
         delete hf;
     
         setProgress(50);
-        *Log << "Loading list ...";
+        *Log << "Loading the list ...\n";
     /*
     *    convert map format
     */
@@ -2075,43 +2302,61 @@ void WorkerThread::processHosts()
             setProgress(50+(t++)*50/count);
         }
     }
-    *Log << "end.";
-    setProgress(100);            
-            
-    
-}
-void WorkerThread::processDNS()
-{
-    *Log << "Discovery method:"
-         << "Import DNS zone";
-    
-    *Log << "end.";
-    setProgress(100);    
-}
-void WorkerThread::processSNMP()
-{
-    *Log << "Discovery method:"
-         << "Perform network discovery using SNMP";
-    
-    *Log << "end.";
+    *Log << "done.\n";
     setProgress(100);
-}
 
+    done();
+}
 
-void WorkerThread::run()
+// ================================================================
+
+ConfigImport::ConfigImport(string *b, const std::string &p) : WorkerThread()
+{
+    buffer = b;
+    platform = p;
+}
+
+ConfigImport::~ConfigImport()
 {
-    switch (Task)
+    if (imp)  delete imp;
+    if (buffer) delete buffer;
+}
+
+void ConfigImport::run()
+{
+    *Log << "Discovery method:"
+         << "Import firewall configuration. \n";
+
+    std::istringstream instream(*buffer);
+    imp = NULL;
+    if (platform == "iosacl") imp = new IOSImporter(om->getCurrentLib(),
+                                                    instream,
+                                                    Log);
+    if (platform == "iptables") imp = new IPTImporter(om->getCurrentLib(),
+                                                      instream,
+                                                      Log);
+
+    // add other platforms here when available
+
+    if (imp)
     {
-        case BT_NONE: { break;}
-        case BT_HOSTS: 
+        try
+        {
+            imp->run();
+        } catch(ImporterException &e)
         {
-            processHosts();
-            break;
+            last_error = e.toString().c_str();
+            *Log << "Parser error:\n";
+            *Log << e.toString() << "\n";
         }
-            
-        case BT_DNS: { break;}
-        case BT_SNMP: { break;}
-                      
+
+        if (fwbdebug)
+            qDebug(imp->getDebugOutput().c_str());
+
+    } else
+    {
+        *Log << "Can not import configuration for choosen platform\n";
     }
+    
     done();
 }
