@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.cpp,v 1.102 2007/04/25 05:00:36 vkurland Exp $
+  $Id: instDialog.cpp,v 1.108 2007/07/13 05:32:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -26,11 +26,13 @@
 #include "../../config.h"
 #include "global.h"
 #include "utils.h"
+#include "utils_no_qt.h"
 
 #include "instDialog.h"
 #include "FWBSettings.h"
 #include "SSHUnx.h"
 #include "SSHPIX.h"
+#include "SSHIOS.h"
 #include "ObjectManipulator.h"
 #include "FWWindow.h"
 #include "InstallFirewallViewItem.h"
@@ -189,102 +191,44 @@ void instDialog::prepareInstConf(Firewall *fw)
     
 }
 
-void instDialog::prepareInstallerOptions(bool fPix)
+void instDialog::prepareInstallerOptions()
 {
     if (fwbdebug) qDebug("instDialog::prepareInstallerOptions");
     ready=false;
     activationCommandDone=false;
-
+    FWOptions  *fwopt = cnf.fwobj->getOptionsObject();
 
     fwb_prompt="--**--**--";
     session = NULL;
 
-    dlg->pwd->setEchoMode( QLineEdit::Password );
-    dlg->epwd->setEchoMode( QLineEdit::Password );
+    cnf.batchInstall = batchInstall->isChecked();
 
-    QString fwname = QString::fromUtf8((*opListIterator)->getName().c_str());
-    if (!batchInstall->isChecked())
-        dlg->dialogTitleLine->setText(
-            QString("<p align=\"center\"><b><font size=\"+2\">")+
-            tr("Install options for firewall '%1'").arg(fwname)+
-            QString("</font></b></p>")
-        );
-    cnf.incremental=st->readBoolEntry("/FirewallBuilder2/Installer/incr"    );
-    cnf.save_diff  =st->readBoolEntry("/FirewallBuilder2/Installer/savediff");
-    cnf.saveStandby=st->readBoolEntry("/FirewallBuilder2/Installer/saveStandby");
-    cnf.dry_run    =st->readBoolEntry("/FirewallBuilder2/Installer/dryrun"  );
-    cnf.quiet      =st->readBoolEntry("/FirewallBuilder2/Installer/quiet"   );
-    cnf.verbose    =st->readBoolEntry("/FirewallBuilder2/Installer/verbose" );
-    cnf.stripComments=st->readBoolEntry("/FirewallBuilder2/Installer/stripComments" );
-    cnf.compressScript=st->readBoolEntry("/FirewallBuilder2/Installer/compressScript" );
-    cnf.copyFWB     =st->readBoolEntry("/FirewallBuilder2/Installer/copyFWB" );
-    cnf.testRun     =st->readBoolEntry("/FirewallBuilder2/Installer/testRun" );
-    cnf.rollback    =st->readBoolEntry("/FirewallBuilder2/Installer/rollback" );
-    cnf.rollbackTime=st->readNumEntry("/FirewallBuilder2/Installer/rollbackTime");
+    cnf.incremental    = st->readBoolEntry("/FirewallBuilder2/Installer/incr"    );
+    cnf.save_diff      = st->readBoolEntry("/FirewallBuilder2/Installer/savediff");
+    cnf.saveStandby    = st->readBoolEntry("/FirewallBuilder2/Installer/saveStandby");
+    cnf.dry_run        = st->readBoolEntry("/FirewallBuilder2/Installer/dryrun"  );
+    cnf.quiet          = st->readBoolEntry("/FirewallBuilder2/Installer/quiet"   );
+    cnf.verbose        = st->readBoolEntry("/FirewallBuilder2/Installer/verbose" );
+    cnf.stripComments  = st->readBoolEntry("/FirewallBuilder2/Installer/stripComments" );
+    cnf.compressScript = st->readBoolEntry("/FirewallBuilder2/Installer/compressScript" );
+    cnf.copyFWB        = st->readBoolEntry("/FirewallBuilder2/Installer/copyFWB" );
+    cnf.testRun        = st->readBoolEntry("/FirewallBuilder2/Installer/testRun" );
+    cnf.rollback       = st->readBoolEntry("/FirewallBuilder2/Installer/rollback" );
+    cnf.rollbackTime   = st->readNumEntry("/FirewallBuilder2/Installer/rollbackTime");
+    cnf.cancelRollbackIfSuccess =
+        st->readBoolEntry("/FirewallBuilder2/Installer/canceRollbackIfSuccess" );
 /* TODO: set cnf.pgm to ssh path here */
 
-    QString platform=cnf.fwobj->getStr("platform").c_str();
-
-    dlg->uname->setFocus();
-    dlg->uname->setText( cnf.user );
-    dlg->incr->setChecked( cnf.incremental );
-    dlg->test->setChecked( cnf.dry_run );
-    dlg->backupConfigFile->setText( cnf.backup_file );
-    dlg->saveDiff->setChecked( cnf.save_diff );
-    dlg->saveStandby->setChecked( cnf.saveStandby );
-    dlg->altAddress->setText( cnf.maddr );
-    dlg->quiet->setChecked( cnf.quiet );
-    dlg->verbose->setChecked( cnf.verbose );
-    dlg->stripComments->setChecked( cnf.stripComments );
-    dlg->compressScript->setChecked( cnf.compressScript );
-    dlg->copyFWB->setChecked( cnf.copyFWB );
-    dlg->testRun->setChecked( cnf.testRun );
-    dlg->rollback->setChecked( cnf.rollback );
-    dlg->rollbackTime->setValue( cnf.rollbackTime );
-
-    bool f = dlg->testRun->isChecked();
-    if (cnf.fwobj->getStr("host_OS")=="pix_os" || cnf.fwobj->getStr("host_OS")=="fwsm_os")
-    {
-        dlg->rollback->setChecked( false );
-        f=false;
-    }
-
-    dlg->updateRollback();
+    QString platform = cnf.fwobj->getStr("platform").c_str();
     
-    QSize pix_options_frame_size = dlg->PIXgroupBox->sizeHint();
+    //bool f = dlg->testRun->isChecked();
+
+    //QSize pix_options_frame_size = dlg->PIXgroupBox->sizeHint();
 
     cnf.rollbackTimeUnit=
         Resources::getTargetOptionStr(cnf.fwobj->getStr("host_OS"),
                                       "activation/timeout_units").c_str();
-    if ((platform!="pix" && platform!="fwsm" && !batchInstall->isChecked()) ||
-        (!fPix && batchInstall->isChecked()) )
-    {
-        dlg->hidePIXOptions();
-    }
 
-/* hide anyway, diff does not work for pix 6.3(3) */
-    //dlg->hideOption( dlg->saveDiff );
-    dlg->saveDiff->hide();
-
-    //progressBar->hide();
-    //dlg->hideOption( dlg->stripComments );
-    dlg->stripComments->hide(); 
-    //dlg->compressScript->hide();
-
-    //if (platform=="pix" || platform=="fwsm")
-    //{
-    //    progressBar->show();
-    //    stripComments->show();
-    //}
-
-    if (cnf.fwobj->getStr("host_OS")!="linksys")
-    {
-        dlg->compressScript->hide();
-        //dlg->hideOption( dlg->compressScript );
-    }
-
-    //dlg->adjustSize();
-    //dlg->setFixedHeight( dlg->minimumSizeHint().height() );
 
 /* we initialize these in FWBSettings constructor on Unix, but do not
  * do it on Windows since there is no standard ssh package there. User
@@ -292,7 +236,7 @@ void instDialog::prepareInstallerOptions(bool fPix)
  * they can't use installer
  */
 
-    ssh=st->getSSHPath();
+    ssh = st->getSSHPath();
 
 /* as of v2.0.3, build 437, incremental install actually installs only
  * ACL and nat commands on PIX. It does not use fwb_pix_diff so there
@@ -308,17 +252,19 @@ void instDialog::prepareInstallerOptions(bool fPix)
         saveDiff->setEnabled(false);
     }
 */
-
+        
     try
     {
         if (cnf.fwobj!=NULL && ! cnf.fwbfile.isEmpty())
         {
-            cnf.maddr = cnf.fwobj->getManagementAddress().toString().c_str();
-//            altAddress->setText( cnf.maddr );
+            QString aaddr = fwopt->getStr("altAddress").c_str();
+            if (!aaddr.isEmpty())
+                cnf.maddr = aaddr;
+            else
+                cnf.maddr =
+                    cnf.fwobj->getManagementAddress().toString().c_str();
         }
 
-        
-
         setReady(true);
 
     } catch(FWException &ex)
@@ -339,9 +285,10 @@ void instDialog::prepareInstallerOptions(bool fPix)
         //showPage( page(1) );
         currentLog->append( QObject::tr("Unsupported exception") );
     }
-    
+
 }
 
+
 void instDialog::storeInstallerOptions()
 {
     st->writeEntry("/FirewallBuilder2/Installer/incr",    cnf.incremental);
@@ -356,6 +303,8 @@ void instDialog::storeInstallerOptions()
     st->writeEntry("/FirewallBuilder2/Installer/testRun",  cnf.testRun);
     st->writeEntry("/FirewallBuilder2/Installer/rollback", cnf.rollback);
     st->writeEntry("/FirewallBuilder2/Installer/rollbackTime", cnf.rollbackTime);
+    st->writeEntry("/FirewallBuilder2/Installer/canceRollbackIfSuccess", 
+      cnf.cancelRollbackIfSuccess);
 }
 
 void instDialog::append(const QString &s)
@@ -519,105 +468,101 @@ void instDialog::selected(const QString &title)
 {
     if (fwbdebug) qDebug("instDialog::selected");
     int p = indexOf( currentPage() );
-    if (fwbdebug) qDebug(QString("to page: %1  from page: %2").arg(p).arg(lastPage));
+    if (fwbdebug)
+        qDebug(QString("to page: %1  from page: %2").arg(p).arg(lastPage));
 
     switch (p)
     {
-        case 0: // select firewalls for compiling and installing
-            {
-                if (lastPage<0) fillCompileSelectList();
-                setNextEnabled(page(0), isTableHasChecked());
-                break;   
-            }
-        case 1: // compiling (installing) firewalls
-            {
-                setBackEnabled(page(1),false);
-                setNextEnabled(page(1),false);
-                if (lastPage<1)
-                { // starting process
-                    fillCompileOpList();
-                    opListIterator=opList.begin();
-                    if (opList.size()==0)
-                    {// there are no firewalls for compilation
-                     // what about installation?
+    case 0: // select firewalls for compiling and installing
+    {
+        if (lastPage<0) fillCompileSelectList();
+        setNextEnabled(page(0), isTableHasChecked());
+        break;   
+    }
+    case 1: // compiling (installing) firewalls
+    {
+        setBackEnabled(page(1),false);
+        setNextEnabled(page(1),false);
+        if (lastPage<1)
+        { // starting process
+            fillCompileOpList();
+            opListIterator=opList.begin();
+            if (opList.size()==0)
+            {// there are no firewalls for compilation
+                // what about installation?
 
-                        compileFlag=false;
+                compileFlag=false;
                      
-                        fillInstallOpList();
-                        opListIterator=opList.begin();
-                        if(opList.size()==0)
-                        {//there are no firewalls for installation
-                            showPage(page(2));
-                        }
-                        else
-                        {
-                            lastPage=p;
-                            initInstall();
-                            installSelected();
-                            if (stopProcessFlag) return;
-                        }
-                    }
-                    else
-                    {
-                        compileSelected();
-                    }
+                fillInstallOpList();
+                opListIterator=opList.begin();
+                if(opList.size()==0)
+                {//there are no firewalls for installation
+                    showPage(page(2));
                 }
                 else
                 {
-                    if (opList.size()==0)
-                    {
-                        showPage(page(2));
-                    }
-                    else
-                    {
-                        installSelected();
-                        if (stopProcessFlag) return;
-                    }
-                }
-                
-                break;
-            }
-        case 2: // fin
-            {
-                setBackEnabled(page(2),false);
-                if (compileFlag && operation==BATCH_INSTALL) 
-                {
-                    fillInstallOpList();
+                    lastPage=p;
                     initInstall();
-                    compileFlag=false;
-                    if (opList.size()>0)
-                    {
-                        opListIterator=opList.begin();
-                        showPage(page(1));
-                    }
-                    else
-                    {
-                        fillLastList();
-                    }
-                    break;
+                    installSelected();
+                    if (stopProcessFlag) return;
                 }
-
-                if (
-                        !compileFlag && 
-                        opList.end()!=opListIterator && 
-                        operation==BATCH_INSTALL &&
-                        !stopProcessFlag)
-                {
-                    showPage(page(1));
-                    break;
-                }
-                fillLastList();
-                break;
             }
-        default:
+            else
             {
-                
+                compileSelected();
             }
+        }
+        else
+        {
+            if (opList.size()==0)
+            {
+                showPage(page(2));
+            }
+            else
+            {
+                installSelected();
+                if (stopProcessFlag) return;
+            }
+        }
+                
+        break;
+    }
+    case 2: // fin
+    {
+        setBackEnabled(page(2),false);
+        if (compileFlag && operation==BATCH_INSTALL) 
+        {
+            fillInstallOpList();
+            initInstall();
+            compileFlag=false;
+            if (opList.size()>0)
+            {
+                opListIterator=opList.begin();
+                showPage(page(1));
+            }
+            else
+            {
+                fillLastList();
+            }
+            break;
+        }
 
-            
+        if (
+            !compileFlag && 
+            opList.end()!=opListIterator && 
+            operation==BATCH_INSTALL &&
+            !stopProcessFlag)
+        {
+            showPage(page(1));
+            break;
+        }
+        fillLastList();
+        break;
+    }
+    default: { }
     }
 
-    lastPage=indexOf( currentPage() );
+    lastPage = indexOf( currentPage() );
 }
 
 QString instDialog::getFullPath(instConf &cnf, const QString &file )
@@ -633,131 +578,135 @@ bool instDialog::doInstallPage()
 /* change of the page when flag ready is 'true' means we should start
  * operation */
 
-        cnf.incremental = dlg->incr->isChecked();
-        cnf.dry_run     = dlg->test->isChecked();
-        cnf.backup_file = dlg->backupConfigFile->text();
-        cnf.backup      = !cnf.backup_file.isEmpty();
-        cnf.save_diff   = dlg->saveDiff->isChecked();
-        cnf.saveStandby = dlg->saveStandby->isChecked();
-        QString aaddr    = dlg->altAddress->text();
-	if (!aaddr.isEmpty())
-	{
+    cnf.incremental = dlg->incr->isChecked();
+    cnf.dry_run     = dlg->test->isChecked();
+    cnf.backup_file = dlg->backupConfigFile->text();
+    cnf.backup      = !cnf.backup_file.isEmpty();
+    cnf.save_diff   = dlg->saveDiff->isChecked();
+    cnf.saveStandby = dlg->saveStandby->isChecked();
+    QString aaddr    = dlg->altAddress->text();
+    if (!aaddr.isEmpty())
+    {
 /* alternative address can also be putty session name. In any case,
  * leave it up to ssh to resolve it and signal an error if it can't be
  * resolved ( Putty session name does not have to be in DNS at all ).
  */
-            cnf.maddr = aaddr;
-	} else
-            cnf.maddr = cnf.fwobj->getManagementAddress().toString().c_str();
+        cnf.maddr = aaddr;
+    } else
+        cnf.maddr = cnf.fwobj->getManagementAddress().toString().c_str();
 
 //        cnf.maddr         = altAddress->text();
-        cnf.user          = dlg->uname->text();
-        cnf.pwd           = dlg->pwd->text();
-        cnf.epwd          = dlg->epwd->text();
-        cnf.quiet         = dlg->quiet->isChecked();
-        cnf.verbose       = dlg->verbose->isChecked();
-        cnf.stripComments = dlg->stripComments->isChecked();
-        cnf.compressScript= dlg->compressScript->isChecked();
-        cnf.copyFWB       = dlg->copyFWB->isChecked();
-        cnf.testRun       = dlg->testRun->isChecked();
-        cnf.rollback      = dlg->rollback->isChecked();
-        cnf.rollbackTime  = dlg->rollbackTime->value();
+    cnf.user          = dlg->uname->text();
+    cnf.pwd           = dlg->pwd->text();
+    cnf.epwd          = dlg->epwd->text();
+    cnf.quiet         = dlg->quiet->isChecked();
+    cnf.verbose       = dlg->verbose->isChecked();
+    cnf.stripComments = dlg->stripComments->isChecked();
+    cnf.compressScript= dlg->compressScript->isChecked();
+    cnf.copyFWB       = dlg->copyFWB->isChecked();
+    cnf.testRun       = dlg->testRun->isChecked();
+    cnf.rollback      = dlg->rollback->isChecked();
+    cnf.rollbackTime  = dlg->rollbackTime->value();
+    cnf.cancelRollbackIfSuccess = dlg->cancelRollbackIfSuccess->isChecked();
 
-        storeInstallerOptions();
+    storeInstallerOptions();
 /* check for a common error when multiple interfaces are marked as
  * 'management'
  */
-        int nmi = 0;
-        list<FWObject*> ll = cnf.fwobj->getByType(Interface::TYPENAME);
-        for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++)
-        {
-            Interface *intf = Interface::cast( *i );
-            if (intf->isManagement()) nmi++;
-        }
-        if (nmi>1)
-        {
-	   addToLog(
-		QObject::tr("Only one interface of the firewall '%1' must be marked as management interface.")
-		.arg(cnf.fwobj->getName().c_str()).latin1() );
-	    return false;
-        }
-        if (nmi==0)
-        {
-	   addToLog(
-		QObject::tr("One of the interfaces of the firewall '%1' must be marked as management interface.")
-		.arg(cnf.fwobj->getName().c_str()).latin1() );
-	    return false;
-        }
-        if ((cnf.maddr == "" || cnf.maddr == "0.0.0.0") &&
-            dlg->altAddress->text().isEmpty())
-	{
-	   addToLog(
-               QObject::tr("Management interface does not have IP address, can not communicate with the firewall.") );
-	    return false;
-	}
+    int nmi = 0;
+    list<FWObject*> ll = cnf.fwobj->getByType(Interface::TYPENAME);
+    for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++)
+    {
+        Interface *intf = Interface::cast( *i );
+        if (intf->isManagement()) nmi++;
+    }
+    if (nmi>1)
+    {
+        addToLog(
+            QObject::tr("Only one interface of the firewall '%1' must be marked as management interface.")
+            .arg(cnf.fwobj->getName().c_str()).latin1() );
+        return false;
+    }
+    if (nmi==0)
+    {
+        addToLog(
+            QObject::tr("One of the interfaces of the firewall '%1' must be marked as management interface.")
+            .arg(cnf.fwobj->getName().c_str()).latin1() );
+        return false;
+    }
+    if ((cnf.maddr == "" || cnf.maddr == "0.0.0.0") &&
+        dlg->altAddress->text().isEmpty())
+    {
+        addToLog(
+            QObject::tr("Management interface does not have IP address, can not communicate with the firewall.") );
+        return false;
+    }
 
-        confScript="";
+    confScript="";
 
-        confFiles.clear();
-        if (cnf.copyFWB)
-        {
-            QFileInfo fwbfile_base(cnf.fwbfile);
+    confFiles.clear();
+    if (cnf.copyFWB)
+    {
+        QFileInfo fwbfile_base(cnf.fwbfile);
             
-            if (fwbdebug)
-                qDebug( QString("Will copy data file: %1").arg(fwbfile_base.fileName()));
+        if (fwbdebug)
+            qDebug( QString("Will copy data file: %1").arg(fwbfile_base.fileName()));
 
-            confFiles.push_back( fwbfile_base.fileName() );
-        }
+        confFiles.push_back( fwbfile_base.fileName() );
+    }
 /* read manifest from the conf file */
 
-        QString conffile_path = getFullPath(cnf,cnf.conffile);
-        QFile cf( conffile_path  );
-        if (cf.open( IO_ReadOnly ) )
+    QString conffile_path = getFullPath(cnf,cnf.conffile);
+    QFile cf( conffile_path  );
+    if (cf.open( IO_ReadOnly ) )
+    {
+        QTextStream stream(&cf);
+        QString line;
+        while (!stream.eof())
         {
-            QTextStream stream(&cf);
-            QString line;
-            while (!stream.eof())
+            line    = stream.readLine();
+            int pos = -1;
+            if ( (pos=line.find(MANIFEST_MARKER))!=-1 )
             {
-                line    = stream.readLine();
-                int pos = -1;
-                if ( (pos=line.find(MANIFEST_MARKER))!=-1 )
-                {
-                    int n = pos + QString(MANIFEST_MARKER).length();
+                int n = pos + QString(MANIFEST_MARKER).length();
 //                if (line[n]=='*')  confScript=line.mid(n+2);
 //                else               confFiles.push_back( line.mid(n+2) );
 
-                    confFiles.push_back( line.mid(n+2) );
+                confFiles.push_back( line.mid(n+2) );
 
-                    if (fwbdebug)
-                        qDebug("instDialog: adding %c %s",
-                               line[n].latin1(),
-                               line.mid(n+2).ascii());
+                if (fwbdebug)
+                    qDebug("instDialog: adding %c %s",
+                           line[n].latin1(),
+                           line.mid(n+2).ascii());
 
-                }
-                line = "";
             }
-            cf.close();
-        } else
-        {
-            QMessageBox::critical(this, "Firewall Builder",
-                 tr("File %1 not found.").arg(conffile_path),
-                                  tr("&Continue") );
-            return false;
+            line = "";
         }
+        cf.close();
+    } else
+    {
+        QMessageBox::critical(this, "Firewall Builder",
+                              tr("File %1 not found.").arg(conffile_path),
+                              tr("&Continue") );
+        return false;
+    }
 
 //    if (confScript.isEmpty()) confScript=cnf.conffile;
-        if (confFiles.size()==0) confFiles.push_back(cnf.conffile);
+    if (confFiles.size()==0) confFiles.push_back(cnf.conffile);
 
-        currentProgressBar->setProgress(0);
+    currentProgressBar->setProgress(0);
 
-        summary();
+    summary();
 
-        QStringList  args;
+    QStringList  args;
 
-        if (cnf.fwobj->getStr("platform")=="pix" || cnf.fwobj->getStr("platform")=="fwsm")
-        {
+    if (cnf.fwobj->getStr("platform")=="pix" ||
+        cnf.fwobj->getStr("platform")=="fwsm" ||
+        cnf.fwobj->getStr("platform")=="iosacl"
+    )
+    {
 #ifdef _WIN32
-            args.push_back(ssh);
+        args.push_back(ssh);
 
 /*
  * putty ignores protocol and port specified in the session file if
@@ -768,64 +717,137 @@ bool instDialog::doInstallPage()
  * use current Windows account name to log in to the firewall and this
  * is unlikely to work anyway. This seems to be a decent workaround.
  */
-            if (!cnf.user.isEmpty() &&
-                ssh.find("plink.exe")!=-1) args.push_back("-ssh");
-
+        if (!cnf.user.isEmpty() && ssh.find("plink.exe")!=-1)
+        {
+            args.push_back("-ssh");
             args.push_back("-pw");
             args.push_back(cnf.pwd);
-#else
-            args.push_back(argv0.c_str());
-            args.push_back("-X");   // fwbuilder works as ssh wrapper
-//            args.push_back("-d");
-            args.push_back("-t");
-            args.push_back("-t");
-#endif
-            if (!cnf.sshArgs.isEmpty())
-                args += QStringList::split(" ",cnf.sshArgs);
+        }
 
-            if (cnf.verbose) args.push_back("-v");
-            if (!cnf.user.isEmpty())
-            {
-                args.push_back("-l");
-                args.push_back(cnf.user);
-                args.push_back(cnf.maddr);
-            } else
-                args.push_back(cnf.maddr);
+        if (!cnf.sshArgs.isEmpty())
+            args += QStringList::split(" ",cnf.sshArgs);
 
-            if (cnf.verbose) displayCommand(args);
-
-            phase=1;
-
-            runSSH( new SSHPIX(this,
-                               cnf.fwobj->getName().c_str(),
-                               args,
-                               cnf.pwd,
-                               cnf.epwd,
-                               list<string>()) );
-        } else
+        if (cnf.verbose) args.push_back("-v");
+        if (!cnf.user.isEmpty())
         {
+            args.push_back("-l");
+            args.push_back(cnf.user);
+            args.push_back(cnf.maddr);
+        } else
+            args.push_back(cnf.maddr);
+#else
+        args.push_back(argv0.c_str());
+        args.push_back("-X");   // fwbuilder works as ssh wrapper
+//            args.push_back("-d");
+        args.push_back("-t");
+        args.push_back("-t");
+
+        if (!cnf.sshArgs.isEmpty())
+            args += QStringList::split(" ",cnf.sshArgs);
+
+        if (cnf.verbose) args.push_back("-v");
+        if (!cnf.user.isEmpty())
+        {
+            args.push_back("-l");
+            args.push_back(cnf.user);
+            args.push_back(cnf.maddr);
+        } else
+            args.push_back(cnf.maddr);
+#endif
+
+        if (cnf.verbose) displayCommand(args);
+
+        phase=1;
+
+        SSHPIX *ssh_object = NULL;
+        if (cnf.fwobj->getStr("platform")=="pix" ||
+            cnf.fwobj->getStr("platform")=="fwsm")
+        {
+            ssh_object = new SSHPIX(this,
+                                    cnf.fwobj->getName().c_str(),
+                                    args,
+                                    cnf.pwd,
+                                    cnf.epwd,
+                                    list<string>());
+        } else   // ios
+        {
+            ssh_object = new SSHIOS(this,
+                                    cnf.fwobj->getName().c_str(),
+                                    args,
+                                    cnf.pwd,
+                                    cnf.epwd,
+                                    list<string>());
+        }
+
+        /*
+         * TODO:
+         * the structure of scriptlets (command templates) for PIX and
+         * IOS is nice and generic, it uses generalized "pre_config"
+         * and "post_config" hooks in SSHPIX / SSHIOS classes. Need to
+         * do the same for Unix firewalls.
+         */
+        
+        QString cmd = "";
+        QStringList pre_config_commands;
+        QStringList post_config_commands;
+
+        cmd = cnf.getCmdFromResource("pre_config_commands");
+        pre_config_commands = QStringList::split("\n", replaceMacrosInCommand(cmd) );
+
+        if (cnf.rollback)
+        {
+            cmd = cnf.getCmdFromResource("schedule_rollback");
+            pre_config_commands = pre_config_commands +
+                QStringList::split("\n", replaceMacrosInCommand(cmd));
+        }
+
+        cmd = cnf.getCmdFromResource("post_config_commands");
+        post_config_commands = QStringList::split("\n", replaceMacrosInCommand(cmd) );
+
+        if (cnf.cancelRollbackIfSuccess)
+        {
+            cmd = cnf.getCmdFromResource("cancel_rollback");
+            post_config_commands = post_config_commands +
+                QStringList::split("\n", replaceMacrosInCommand(cmd));
+        }
+
+        if (cnf.saveStandby)
+        {
+            cmd = cnf.getCmdFromResource("save_standby");
+            post_config_commands = post_config_commands +
+                QStringList::split("\n", replaceMacrosInCommand(cmd));
+        }
+
+        ssh_object->loadPreConfigCommands( pre_config_commands );
+        ssh_object->loadPostConfigCommands( post_config_commands );
+
+        runSSH(ssh_object);
+        return true;
+    }
+    // all other Unix-based platforms
+
 /*
  * if user requested test run, store firewall script in a temp file.
  * Always store it in a temp file on linksys
  */
-            QString s;
+    QString s;
 
 /* user_can_change_install_dir */
-            bool uccid=Resources::getTargetOptionBool(
-                cnf.fwobj->getStr("host_OS"),"user_can_change_install_dir");
+    bool uccid=Resources::getTargetOptionBool(
+        cnf.fwobj->getStr("host_OS"),"user_can_change_install_dir");
 
-            if (uccid)
-                s=cnf.fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
+    if (uccid)
+        s=cnf.fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
 
-            if (s.isEmpty()) s=Resources::getTargetOptionStr(
-                cnf.fwobj->getStr("host_OS"),
-                "activation/fwdir").c_str();
+    if (s.isEmpty()) s=Resources::getTargetOptionStr(
+        cnf.fwobj->getStr("host_OS"),
+        "activation/fwdir").c_str();
 
-            cnf.fwdir     = s;
+    cnf.fwdir     = s;
 
-            continueRun();
-        }
-        return true;
+    continueRun();
+
+    return true;
 }
 
 void instDialog::resetInstallSSHSession()
@@ -887,6 +909,11 @@ QString instDialog::getActivationCmd()
 
     cmd=Resources::getTargetOptionStr(cnf.fwobj->getStr("host_OS"),
                                       optpath).c_str();
+    return replaceMacrosInCommand(cmd);
+}
+
+QString instDialog::replaceMacrosInCommand(const QString &ocmd)
+{
 
 /* replace macros in activation command:
  *
@@ -895,6 +922,8 @@ QString instDialog::getActivationCmd()
  * check if cnf.conffile is a full path. If it is, strip the path part
  * and use only the file name for %FWSCRIPT%
  */
+    QString cmd = ocmd;
+
     QString clean_conffile = cnf.conffile.section(QDir::separator(),-1);
     if (fwbdebug)
     {
@@ -914,7 +943,6 @@ QString instDialog::getActivationCmd()
     QString rbt;
     rbt.setNum(r);
     cmd.replace("%RBTIMEOUT%",rbt);
-
     return cmd;
 }
 
@@ -974,11 +1002,13 @@ void instDialog::initiateCopy(const QString &file)
 #ifdef _WIN32
     args.push_back(ssh);
 
-    if (!cnf.user.isEmpty() &&
-        ssh.find("plink.exe")!=-1) args.push_back("-ssh");
+    if (!cnf.user.isEmpty() && ssh.find("plink.exe")!=-1)
+    {
+        args.push_back("-ssh");
+        args.push_back("-pw");
+        args.push_back(cnf.pwd);
+    }
 
-    args.push_back("-pw");
-    args.push_back(cnf.pwd);
 #else
     args.push_back(argv0.c_str());
     args.push_back("-X");   // fwbuilder works as ssh wrapper
@@ -1021,19 +1051,25 @@ void instDialog::initiateCopy(const QString &file)
  * check if cnf.conffile is a full path. If it is, strip the path part
  * and use only the file name for %FWSCRIPT%
  */
+    QString file_name_no_spaces = file;
+    if (file_name_no_spaces.find(" ")!=-1)
+    {
+        file_name_no_spaces.replace(" ","\\ ");
+    }
+
     QString clean_file = file_with_path.section(QDir::separator(),-1);
     if (fwbdebug)
     {
         qDebug("Macro substitutions:");
         qDebug(QString("  cnf.conffile=%1").arg(cnf.conffile));
-        qDebug(QString("  %%FWSCRIPT%%=%1").arg(file));
+        qDebug(QString("  %%FWSCRIPT%%=%1").arg(file_name_no_spaces));
         qDebug(QString("  %%FWDIR%%=%1").arg(cnf.fwdir));
     }
 
     cmd.replace("\n","");
-    cmd.replace("%FWSCRIPT%",file);
-    cmd.replace("%FWDIR%",cnf.fwdir);
-    cmd.replace("%FWBPROMPT%",fwb_prompt);
+    cmd.replace("%FWSCRIPT%", file_name_no_spaces);
+    cmd.replace("%FWDIR%", cnf.fwdir);
+    cmd.replace("%FWBPROMPT%", fwb_prompt);
 
     args.push_back(cmd);
         
@@ -1127,7 +1163,10 @@ void instDialog::continueRun()
         return;
     }
 
-    if (cnf.fwobj->getStr("platform")=="pix" || cnf.fwobj->getStr("platform")=="fwsm")
+    if (cnf.fwobj->getStr("platform")=="pix" ||
+        cnf.fwobj->getStr("platform")=="fwsm" ||
+        cnf.fwobj->getStr("platform")=="iosacl"
+    )
     {
         finishInstall();
         return;
@@ -2004,7 +2043,6 @@ void instDialog::fillLastList()
     
     QListViewItem *item;
     Firewall* f;
-    QDateTime dt;
     t_procMess m;
     
     for (map<libfwbuilder::Firewall *, t_procMess>::iterator i=processedFirewalls.begin();
@@ -2136,14 +2174,13 @@ void instDialog::installSelected()
         appendRich(QString("<b>")+currentSearchString+
                     QString::fromUtf8((*opListIterator)->getName().c_str())+"</b>");
         appendRich("\n");
-        qApp->processEvents();
+        //qApp->processEvents();
         
     
-        if(testFirewall(*opListIterator))
+        if (testFirewall(*opListIterator))
         {
             opListMapping[*opListIterator]->setText(1,tr("Installing ..."));
-            qApp->processEvents();
-            
+            //qApp->processEvents();
             
             if (customScriptFlag && fCustInst)
             { // custom installer
@@ -2154,7 +2191,8 @@ void instDialog::installSelected()
             { // buildin installer
                 if (fwbdebug) qDebug("buildin installer");
                 // Show options dialog
-                if (!batchInstall->isChecked() || opListIterator==opList.begin())
+                if (!batchInstall->isChecked() ||
+                    opListIterator==opList.begin())
                 {
                     if (dlg) 
                     {
@@ -2162,20 +2200,17 @@ void instDialog::installSelected()
                         dlg=NULL;
                     }
                     
-                    
+                    prepareInstallerOptions();
                     
                     if (batchInstall->isChecked())
                     {
-                        dlg=new instBatchOptionsDialog(this);
-                        
+                        dlg=new instBatchOptionsDialog(this, &cnf);
                     }
                     else
                     {
-                        dlg=new instOptionsDialog(this);
+                        dlg=new instOptionsDialog(this, &cnf);
                     }
                     
-                    prepareInstallerOptions(fPix);
-                    
                     if (dlg->exec()==QDialog::Rejected)
                     {
                         
@@ -2204,7 +2239,7 @@ void instDialog::installSelected()
                     setTitle(page(1),
                        QObject::tr("Installing policy rules on firewall '%1'.").arg(
                                  (*opListIterator)->getName().c_str() ));
-                    qApp->processEvents();
+                    //qApp->processEvents();
                     if (!runInstall(*opListIterator))
                     {
                         if (fwbdebug) qDebug("start error");
@@ -2222,7 +2257,7 @@ void instDialog::installSelected()
                     return;
                 }
             }
-            if(!runInstall(*opListIterator))
+            if (!runInstall(*opListIterator))
             {
                 processedFirewalls[*opListIterator].second="start error";
                 opListMapping[*opListIterator]->setText(1,tr("Failure"));
@@ -2254,7 +2289,9 @@ void instDialog::findFirewallInCompileLog(QListViewItem* item)
     qApp->processEvents();
     fw=findFirewallbyListItem(item);
     procLogDisplay->moveCursor( QTextEdit::MoveEnd , false );
-    procLogDisplay->find(currentSearchString+QString::fromUtf8(fw->getName().c_str()),true,true,true,&p,&i);
+    procLogDisplay->find(
+        currentSearchString + 
+        QString::fromUtf8(fw->getName().c_str()),true,true,true,&p,&i);
 }
 
 Firewall * instDialog::findFirewallbyListItem(QListViewItem *item)
@@ -2421,7 +2458,7 @@ void instDialog::analyseInstallQueue(bool &fPix, bool &fCustInst)
         mgmt=f->getManagementObject();
         pis   = mgmt->getPolicyInstallScript();
         
-        fPix = fPix || f->getStr("platform")=="pix" || f->getStr("platform")=="fwsm";
+        fPix = fPix || f->getStr("platform")=="pix" || f->getStr("platform")=="fwsm" || f->getStr("platform")=="iosacl";
         fCustInst =  fCustInst && !( pis->getCommand()=="" );
         
         if (cnf.fwobj==NULL && !fCustInst)
