@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: iptAdvancedDialog.cpp,v 1.28 2007/01/15 04:37:52 vkurland Exp $
+  $Id: iptAdvancedDialog.cpp,v 1.29 2007/07/07 19:29:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -99,8 +99,10 @@ iptAdvancedDialog::iptAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption(compiler,             fwoptions, "compiler");
     data.registerOption(compilerArgs,         fwoptions, "cmdline");
     data.registerOption(outputFileName,       fwoptions, "output_file");
-    data.registerOption(assumeFwIsPartOfAny,  fwoptions, "firewall_is_part_of_any_and_networks");
-    data.registerOption(acceptSessions,       fwoptions, "accept_new_tcp_with_no_syn");
+    data.registerOption(assumeFwIsPartOfAny,
+                        fwoptions, "firewall_is_part_of_any_and_networks");
+    data.registerOption(acceptSessions,
+                        fwoptions, "accept_new_tcp_with_no_syn");
     data.registerOption(dropInvalid,          fwoptions, "drop_invalid");
     data.registerOption(logInvalid,           fwoptions, "log_invalid");
     data.registerOption(acceptESTBeforeFirst, fwoptions, "accept_established");
@@ -110,22 +112,26 @@ iptAdvancedDialog::iptAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption(localNAT,             fwoptions, "local_nat");
     data.registerOption(clampMSStoMTU,        fwoptions, "clamp_mss_to_mtu");
     data.registerOption(makeTagClassifyTerminating,
-                                              fwoptions, "classify_mark_terminating");
-
+                        fwoptions, "classify_mark_terminating");
+    data.registerOption(skipIPv6,
+                        fwoptions, "no_ipv6_default_policy");
     slm=getActionsOnReject( obj->getStr("platform").c_str() );
     actionOnReject->clear();
     actionOnReject->insertStringList(getScreenNames(slm));
-    data.registerOption( actionOnReject,      fwoptions,"action_on_reject", slm); 
+    data.registerOption( actionOnReject,
+                         fwoptions,"action_on_reject", slm); 
     
     data.registerOption(mgmt_ssh,             fwoptions, "mgmt_ssh"    );
     data.registerOption(mgmt_addr,            fwoptions, "mgmt_addr"    );
-    data.registerOption(addVirtualsforNAT,    fwoptions, "manage_virtual_addr"    );
-    data.registerOption(configureInterfaces,  fwoptions, "configure_interfaces"    );
+    data.registerOption(addVirtualsforNAT,
+                        fwoptions, "manage_virtual_addr"    );
+    data.registerOption(configureInterfaces,
+                        fwoptions, "configure_interfaces"    );
     data.registerOption(iptDebug,             fwoptions, "debug"    );
     data.registerOption(verifyInterfaces,     fwoptions, "verify_interfaces"    );
     data.registerOption(loadModules,          fwoptions, "load_modules"    );
     data.registerOption(iptablesRestoreActivation,
-                                              fwoptions, "use_iptables_restore"    );
+                        fwoptions, "use_iptables_restore"    );
     data.registerOption(ipt_fw_dir,           fwoptions, "firewall_dir"    );
     data.registerOption(ipt_user,             fwoptions, "admUser"    );
     data.registerOption(altAddress,           fwoptions, "altAddress"    );
